@@ -54,9 +54,6 @@ setGeneric("strandMode<-", signature="x",
     function(x, value) standardGeneric("strandMode<-")
 )
 
-setGeneric("first", function(x, ...) standardGeneric("first"))
-setGeneric("last", function(x, ...) standardGeneric("last"))
-
 setGeneric("isProperPair", function(x) standardGeneric("isProperPair"))
 
 
@@ -83,57 +80,52 @@ setMethod("first", "GAlignmentPairs",
             stop("'real.strand' must be TRUE or FALSE")
         if (!identical(invert.strand, FALSE)) {
             msg <- c("Using the 'invert.strand' argument when calling ",
-                     "first() on a GAlignmentPairs object is deprecated.")
-            .Deprecated(msg=wmsg(msg))
-            if (!isTRUEorFALSE(invert.strand))
-                stop("'invert.strand' must be TRUE or FALSE")
-            if (real.strand && invert.strand)
-                stop(wmsg("one of 'real.strand' or 'invert.strand' can ",
-                          "be set to TRUE but not both"))
+                     "first() on a GAlignmentPairs object is defunct.")
+            .Defunct(msg=wmsg(msg))
         }
         ans <- setNames(x@first, names(x))
         if (invert.strand)
-            return(invertRleStrand(ans))
+            return(invertStrand(ans))
         if (real.strand) {
             if (strandMode(x) == 0L) {
                 strand(ans) <- "*"
             } else if (strandMode(x) == 2L) {
-                ans <- invertRleStrand(ans)
+                ans <- invertStrand(ans)
             }
         }
         ans
     }
 )
 
-setMethod("last", "GAlignmentPairs",
+setGeneric("last", function(x, ...) standardGeneric("last"))
+
+setMethod("last", "GAlignmentPairs", function(x, ...) second(x, ...))
+
+setMethod("second", "GAlignmentPairs",
     function(x, real.strand=FALSE, invert.strand=FALSE)
     {
         if (!isTRUEorFALSE(real.strand))
             stop("'real.strand' must be TRUE or FALSE")
         if (!identical(invert.strand, FALSE)) {
             msg <- c("Using the 'invert.strand' argument when calling ",
-                     "last() on a GAlignmentPairs object is deprecated.")
-            .Deprecated(msg=wmsg(msg))
-            if (!isTRUEorFALSE(invert.strand))
-                stop("'invert.strand' must be TRUE or FALSE")
-            if (real.strand && invert.strand)
-                stop(wmsg("one of 'real.strand' or 'invert.strand' can ",
-                          "be set to TRUE but not both"))
+                     "last() on a GAlignmentPairs object is defunct.")
+            .Defunct(msg=wmsg(msg))
         }
         ans <- setNames(x@last, names(x))
         if (invert.strand)
-            return(invertRleStrand(ans))
+            return(invertStrand(ans))
         if (real.strand) {
             if (strandMode(x) == 0L) {
                 strand(ans) <- "*"
             } else if (strandMode(x) == 1L) {
-                ans <- invertRleStrand(ans)
+                ans <- invertStrand(ans)
             }
         }
         ans
     }
 )
 
+          
 setMethod("seqnames", "GAlignmentPairs",
     function(x) seqnames(x@first)
 )
@@ -201,43 +193,11 @@ setReplaceMethod("strand", "GAlignmentPairs",
     function(x, value)
     {
         msg <- c("The strand setter for GAlignmentPairs objects ",
-                 "is deprecated. You can use strandMode() to control the ",
+                 "is defunct. You can use strandMode() to control the ",
                  "behavior of the strand() getter in accordance with the ",
                  "stranded protocol that was used to generate the ",
                  "paired-end data (see '?strandMode').")
-        .Deprecated(msg=wmsg(msg))
-        if (strandMode(x) == 0L)
-            stop("cannot alter the strand of a ", class(GAlignmentPairs),
-                 " object that has its strand mode set to 0")
-        same_strand <- strand(x@first) == strand(x@last)
-        if (strandMode(x) == 1L) {
-            ## Set the strand of the first alignment.
-            strand(x@first) <- value
-            ## Then set the strand of the last alignment to preserve the
-            ## original relationship between first and last strand (i.e. if
-            ## they were the same, they remain the same, if they were opposite,
-            ## they remain opposite).
-            strand(x@last) <- strand(same_strand == (strand(x@first) == "-"))
-        } else {
-            ## Set the strand of the last alignment.
-            strand(x@last) <- value
-            ## Then set the strand of the first alignment to preserve the
-            ## original relationship between first and last strand (i.e. if
-            ## they were the same, they remain the same, if they were opposite,
-            ## they remain opposite).
-            strand(x@first) <- strand(same_strand == (strand(x@last) == "-"))
-        }
-        x
-    }
-)
-
-setReplaceMethod("elementMetadata", "GAlignmentPairs",
-    function(x, ..., value)
-    {
-        value <-
-            GenomicRanges:::normalizeMetadataColumnsReplacementValue(value, x)
-        x@elementMetadata <- value
-        x
+        .Defunct(msg=wmsg(msg))
     }
 )
 
@@ -367,7 +327,7 @@ GAlignmentPairs <- function(first, last,
          NAMES=names,
          first=first, last=last,
          isProperPair=isProperPair,
-         elementMetadata=new("DataFrame", nrows=length(first)),
+         elementMetadata=S4Vectors:::make_zero_col_DataFrame(length(first)),
          check=TRUE)
 }
 
@@ -448,15 +408,15 @@ shrinkByHalf <- function(x)
 {
     if (length(x) %% 2L != 0L)
         stop("'x' must have an even length")
-    x_elt_lens <- elementLengths(x)
-    if (length(x_elt_lens) == 0L) {
+    x_eltNROWS <- elementNROWS(x)
+    if (length(x_eltNROWS) == 0L) {
         ans_nelt1 <- ans_nelt2 <- integer(0)
     } else {
-        ans_nelt1 <- x_elt_lens[c(TRUE, FALSE)]
-        ans_nelt2 <- x_elt_lens[c(FALSE, TRUE)]
+        ans_nelt1 <- x_eltNROWS[c(TRUE, FALSE)]
+        ans_nelt2 <- x_eltNROWS[c(FALSE, TRUE)]
     }
-    ans_elt_lens <- ans_nelt1 + ans_nelt2
-    ans_partitioning <- PartitioningByEnd(cumsum(ans_elt_lens))
+    ans_eltNROWS <- ans_nelt1 + ans_nelt2
+    ans_partitioning <- PartitioningByEnd(cumsum(ans_eltNROWS))
     ans <- relist(x@unlistData, ans_partitioning)
     mcols(ans) <- DataFrame(nelt1=ans_nelt1, nelt2=ans_nelt2)
     ans
@@ -474,8 +434,8 @@ setMethod("grglist", "GAlignmentPairs",
                      "GAlignmentPairs objects *always* returns the ranges ",
                      "\"ordered as in query\". Therefore the ",
                      "'order.as.in.query' argument is now ignored (and ",
-                     "deprecated).")
-            .Deprecated(msg=wmsg(msg))
+                     "defunct).")
+            .Defunct(msg=wmsg(msg))
         }
         x_mcols <- mcols(x)
         if (use.mcols && "query.break" %in% colnames(x_mcols))
@@ -483,10 +443,10 @@ setMethod("grglist", "GAlignmentPairs",
         x_first <- x@first
         x_last <- x@last
         if (strandMode(x) == 1L) {
-            x_last <- invertRleStrand(x@last)
+            x_last <- invertStrand(x@last)
             x_unlisted <- c(x_first, x_last)
         } else if (strandMode(x) == 2L) {
-            x_first <- invertRleStrand(x@first)
+            x_first <- invertStrand(x@first)
             x_unlisted <- c(x_last, x_first)
         }
         ## Not the same as doing 'unlist(x, use.names=FALSE)'.
@@ -512,7 +472,7 @@ setMethod("granges", "GAlignmentPairs",
         if (!isTRUEorFALSE(use.mcols))
             stop("'use.mcols' must be TRUE or FALSE")
         rg <- range(grglist(x))
-        if (!all(elementLengths(rg) == 1L))
+        if (!all(elementNROWS(rg) == 1L))
             stop("For some pairs in 'x', the first and last alignments ",
                  "are not aligned to the same chromosome and strand. ",
                  "Cannot extract a single range for them.")
@@ -533,6 +493,19 @@ setAs("GAlignmentPairs", "GAlignments",
     function(from) unlist(from, use.names=TRUE)
 )
 
+setAs("GAlignmentPairs", "DataFrame", function(from) {
+          firstDF <- as(first(from), "DataFrame")
+          colnames(firstDF) <- paste0(colnames(firstDF), ".first")
+          lastDF <- as(last(from), "DataFrame")
+          colnames(lastDF) <- paste0(colnames(lastDF), ".last")
+          DataFrame(firstDF, lastDF, mcols(from))
+      })
+
+setMethod("as.data.frame", "GAlignmentPairs",
+          function(x, row.names = NULL, optional = FALSE) {
+              as.data.frame(as(x, "DataFrame"), row.names=row.names,
+                            optional=optional)
+          })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### fillJunctionGaps()
@@ -555,8 +528,8 @@ fillJunctionGaps <- function(x)
     half1_partitioning <- PartitioningByEnd(cumsum(query.breaks))
     half1 <- relist(x@unlistData[idx], half1_partitioning)
     half1 <- range(half1)@unlistData
-    half2_eltlens <- elementLengths(x) - query.breaks
-    half2_partitioning <- PartitioningByEnd(cumsum(half2_eltlens))
+    half2_eltNROWS <- elementNROWS(x) - query.breaks
+    half2_partitioning <- PartitioningByEnd(cumsum(half2_eltNROWS))
     half2 <- relist(x@unlistData[-idx], half2_partitioning)
     half2 <- range(half2)@unlistData
     collate_subscript <- S4Vectors:::make_XYZxyz_to_XxYyZz_subscript(length(x))
@@ -701,7 +674,7 @@ combine_GAlignmentPairs_objects <- function(Class, objects,
         noname_idx <- which(has_no_names)
         if (length(noname_idx) != 0L)
             NAMES_slots[noname_idx] <-
-                lapply(elementLengths(objects[noname_idx]), character)
+                lapply(elementNROWS(objects[noname_idx]), character)
         ans_NAMES <- unlist(NAMES_slots, use.names=FALSE)
     }
 
@@ -725,7 +698,7 @@ combine_GAlignmentPairs_objects <- function(Class, objects,
     ## IRanges:::rbind.mcols() for this because the "mcols" slot of a
     ## GAlignmentPairs object is guaranteed to be a DataFrame.
     if (ignore.mcols) {
-        ans_mcols <- new("DataFrame", nrows=length(ans_first))
+        ans_mcols <- S4Vectors:::make_zero_col_DataFrame(length(ans_first))
     } else  {
         mcols_slots <- lapply(objects, function(x) x@elementMetadata)
         ## Will fail if not all the GAlignmentPairs objects in 'objects' have
@@ -770,38 +743,18 @@ setGeneric("right", function(x, ...) standardGeneric("right"))
 setMethod("left", "GAlignmentPairs",
     function(x, ...)
     {
-        msg <- c("The left() and right() getters are deprecated ",
+        msg <- c("The left() and right() getters are defunct ",
                  "for GAlignmentPairs objects.")
-        .Deprecated(msg=wmsg(msg))
-
-        x_first <- x@first
-        x_last <- invertRleStrand(x@last)
-
-        left_is_last <- which(strand(x_first) == "-")
-        idx <- seq_len(length(x))
-        idx[left_is_last] <- idx[left_is_last] + length(x)
-
-        ans <- c(x_first, x_last)[idx]
-        setNames(ans, names(x))
+        .Defunct(msg=wmsg(msg))
     }
 )
 
 setMethod("right", "GAlignmentPairs",
     function(x, ...)
     {
-        msg <- c("The left() and right() getters are deprecated ",
+        msg <- c("The left() and right() getters are defunct ",
                  "for GAlignmentPairs objects.")
-        .Deprecated(msg=wmsg(msg))
-
-        x_first <- x@first
-        x_last <- invertRleStrand(x@last)
-
-        right_is_first <- which(strand(x_first) == "-")
-        idx <- seq_len(length(x))
-        idx[right_is_first] <- idx[right_is_first] + length(x)
-
-        ans <- c(x_last, x_first)[idx]
-        setNames(ans, names(x))
+        .Defunct(msg=wmsg(msg))
     }
 )
 
