@@ -97,7 +97,7 @@ setMethod("qwidth", "GAlignments",
 )
 
 setMethod("njunc", "GAlignments",
-    function(x) {unname(elementLengths(rglist(x))) - 1L}
+    function(x) {unname(elementNROWS(rglist(x))) - 1L}
 )
 
 setMethod("seqinfo", "GAlignments", function(x) x@seqinfo)
@@ -118,60 +118,11 @@ setReplaceMethod("names", "GAlignments",
     }
 )
 
-.normargSeqnamesReplaceValue <- function(x, value, ans.type=c("factor", "Rle"))
-{
-    ans.type <- match.arg(ans.type)
-    if (!is.factor(value)
-     && !is.character(value)
-     && (!is(value, "Rle") || !is.character(runValue(value))
-                              && !is.factor(runValue(value))))
-        stop("'seqnames' value must be a character factor/vector, ",
-             "or a 'character' Rle, or a 'factor' Rle")
-    if (ans.type == "factor") {
-        if (!is.factor(value))
-            value <- as.factor(value)
-    } else if (ans.type == "Rle") {
-        ## We want to return a 'factor' Rle.
-        if (!is(value, "Rle")) {
-            if (!is.factor(value))
-                value <- as.factor(value)
-            value <- Rle(value)
-        } else if (!is.factor(runValue(value))) {
-            runValue(value) <- as.factor(runValue(value))
-        }
-    }
-    if (length(value) != length(x))
-        stop("'seqnames' value must be the same length as the object")
-    value
-}
-
-### 'old_seqnames' and 'new_seqnames' must be 'factor' Rle.
-.getSeqnamesTranslationTable <- function(old_seqnames, new_seqnames)
-{
-    old <- runValue(old_seqnames)
-    new <- runValue(new_seqnames)
-    tmp <- unique(data.frame(old=old, new=new))
-    if (!identical(runLength(old_seqnames), runLength(new_seqnames)) ||
-        anyDuplicated(tmp$old) || anyDuplicated(tmp$new))
-        stop("mapping between old an new 'seqnames' values is not one-to-one")
-    if (isTRUE(all.equal(as.integer(tmp$old), as.integer(tmp$new)))) {
-        tr_table <- levels(new)
-        names(tr_table) <- levels(old)
-    } else {
-        tr_table <- tmp$new
-        names(tr_table) <- tmp$old
-    }
-    tr_table
-}
-
 setReplaceMethod("seqnames", "GAlignments",
     function(x, value)
     {
-        value <- .normargSeqnamesReplaceValue(x, value, ans.type="Rle")
-        tr_table <- .getSeqnamesTranslationTable(seqnames(x), value)
-        x@seqnames <- value
-        seqnames(x@seqinfo) <- tr_table[seqlevels(x)]
-        x
+        value <- GenomicRanges:::.normalize_seqnames_replacement_value(value, x)
+        BiocGenerics:::replaceSlots(x, seqnames=value)
     }
 )
 
@@ -188,16 +139,6 @@ setReplaceMethod("strand", "GAlignments",
     }
 )
 
-setReplaceMethod("elementMetadata", "GAlignments",
-    function(x, ..., value)
-    {
-        value <-
-            GenomicRanges:::normalizeMetadataColumnsReplacementValue(value, x)
-        x@elementMetadata <- value
-        x
-    }
-)
-
 setReplaceMethod("seqinfo", "GAlignments",
     function(x, new2old=NULL, force=FALSE, value)
     {
@@ -242,11 +183,7 @@ setReplaceMethod("seqinfo", "GAlignments",
 
 .valid.GAlignments.seqnames <- function(x)
 {
-    x_seqnames <- seqnames(x)
-    if (!is(x_seqnames, "Rle") || !is.factor(runValue(x_seqnames))
-     || !is.null(names(x_seqnames)) || any(is.na(x_seqnames)))
-        return("'seqnames(x)' must be an unnamed 'factor' Rle with no NAs")
-    if (length(x_seqnames) != length(cigar(x)))
+    if (length(seqnames(x)) != length(cigar(x)))
         return("'seqnames(x)' and 'cigar(x)' must have the same length")
     NULL
 }
@@ -287,6 +224,7 @@ setReplaceMethod("seqinfo", "GAlignments",
 .valid.GAlignments <- function(x)
 {
     c(.valid.GAlignments.names(x),
+      GenomicRanges:::.valid.GenomicRanges.seqnames(x),
       .valid.GAlignments.seqnames(x),
       .valid.GAlignments.start(x),
       .valid.GAlignments.cigar(x),
@@ -343,7 +281,7 @@ GAlignments <- function(seqnames=Rle(factor()), pos=integer(0),
     varlist <- list(...)
     elementMetadata <- 
         if (0L == length(varlist))
-            new("DataFrame", nrows=length(seqnames))
+            S4Vectors:::make_zero_col_DataFrame(length(seqnames))
         else
             do.call(DataFrame, varlist)
     ## Prepare the 'seqinfo' slot.
@@ -414,9 +352,9 @@ setMethod("update", "GAlignments",
 ### Names are propagated via 'x@partitioning' ('x' is a CompressedIRangesList).
 .CompressedIRangesListToGRangesList <- function(x, seqnames, strand, seqinfo)
 {
-    elt_lens <- elementLengths(x)
-    seqnames <- rep.int(seqnames, elt_lens)
-    strand <- rep.int(strand, elt_lens)
+    x_eltNROWS <- elementNROWS(x)
+    seqnames <- rep.int(seqnames, x_eltNROWS)
+    strand <- rep.int(strand, x_eltNROWS)
     unlisted_ans <- GRanges(seqnames=seqnames, ranges=x@unlistData,
                             strand=strand)
     seqinfo(unlisted_ans) <- seqinfo
@@ -483,28 +421,25 @@ setAs("GAlignments", "GRanges", function(from) granges(from, use.mcols=TRUE))
 setAs("GAlignments", "RangesList", function(from) rglist(from, use.mcols=TRUE))
 setAs("GAlignments", "Ranges", function(from) ranges(from))
 
+setAs("GAlignments", "DataFrame", function(from) {
+          DataFrame(seqnames=seqnames(from),
+                    strand=strand(from),
+                    cigar=cigar(from),
+                    qwidth=qwidth(from),
+                    start=start(from),
+                    end=end(from),
+                    width=width(from),
+                    njunc=njunc(from),
+                    mcols(from),
+                    row.names=names(from),
+                    check.names=FALSE)
+      })
+
 setMethod("as.data.frame", "GAlignments",
     function(x, row.names=NULL, optional=FALSE, ...)
     {
-        if (is.null(row.names))
-            row.names <- names(x)
-        else if (!is.character(row.names))
-            stop("'row.names' must be NULL or a character vector")
-        ans <- data.frame(seqnames=as.character(seqnames(x)),
-                          strand=as.character(strand(x)),
-                          cigar=cigar(x),
-                          qwidth=qwidth(x),
-                          start=start(x),
-                          end=end(x),
-                          width=width(x),
-                          njunc=njunc(x),
-                          row.names=row.names,
-                          check.rows=TRUE,
-                          check.names=FALSE,
-                          stringsAsFactors=FALSE)
-        if (ncol(mcols(x)))
-            ans <- cbind(ans, as.data.frame(mcols(x)))
-        return(ans)
+        as.data.frame(as(x, "DataFrame"), row.names=row.names,
+                      optional=optional)
     }
 )
 
@@ -678,7 +613,7 @@ combine_GAlignments_objects <- function(Class, objects,
         noname_idx <- which(has_no_names)
         if (length(noname_idx) != 0L)
             NAMES_slots[noname_idx] <-
-                lapply(elementLengths(objects[noname_idx]), character)
+                lapply(elementNROWS(objects[noname_idx]), character)
         ans_NAMES <- unlist(NAMES_slots, use.names=FALSE)
     }
 
@@ -704,7 +639,7 @@ combine_GAlignments_objects <- function(Class, objects,
     ## IRanges:::rbind.mcols() for this because the "mcols" slot of a
     ## GAlignments object is guaranteed to be a DataFrame.
     if (ignore.mcols) {
-        ans_mcols <- new("DataFrame", nrows=length(ans_start))
+        ans_mcols <- S4Vectors:::make_zero_col_DataFrame(length(ans_start))
     } else  {
         mcols_slots <- lapply(objects, function(x) x@elementMetadata)
         ## Will fail if not all the GAlignments objects in 'objects' have
