@@ -194,10 +194,9 @@ setMethod("grglist", "GAlignmentsList",
         if (!isTRUEorFALSE(use.mcols))
             stop("'use.mcols' must be TRUE or FALSE")
         if (!identical(order.as.in.query, FALSE)) {
-            msg <- c("Starting with BioC 3.2, the 'order.as.in.query' ",
-                     "argument of the \"grglist\" method for ",
-                     "GAlignmentsList objects is deprecated and ignored.")
-            .Deprecated(msg=wmsg(msg))
+            msg <- c("the 'order.as.in.query' argument of the \"grglist\" ",
+                     "method for GAlignmentsList objects is defunct")
+            .Defunct(msg=wmsg(msg))
         }
         if (!isTRUEorFALSE(ignore.strand))
             stop("'ignore.strand' must be TRUE or FALSE")
@@ -225,7 +224,7 @@ setMethod("granges", "GAlignmentsList",
                       "As a consequence, the returned GRanges object ",
                       "is not parallel to 'x'.")
         rg <- range(grglist(x, ignore.strand=ignore.strand))
-        is_one_to_one <- all(elementLengths(rg) == 1L)
+        is_one_to_one <- all(elementNROWS(rg) == 1L)
         if (!is_one_to_one && all(width(x@partitioning) > 0)) {
             if (ignore.strand)
                 warning(msg)
@@ -245,10 +244,9 @@ setMethod("rglist", "GAlignmentsList",
         if (!isTRUEorFALSE(use.mcols))
             stop("'use.mcols' must be TRUE or FALSE")
         if (!identical(order.as.in.query, FALSE)) {
-            msg <- c("Starting with BioC 3.2, the 'order.as.in.query' ",
-                     "argument of the \"rglist\" method for ",
-                     "GAlignmentsList objects is deprecated and ignored.")
-            .Deprecated(msg=wmsg(msg))
+            msg <- c("the 'order.as.in.query' argument of the \"rglist\" ",
+                     "method for GAlignmentsList objects is defunct")
+            .Defunct(msg=wmsg(msg))
         }
         unlisted_x <- unlist(x, use.names=FALSE)
         rgl <- rglist(unlisted_x, drop.D.ranges=drop.D.ranges)
@@ -290,6 +288,19 @@ setAs("GAlignmentPairs", "GAlignmentsList",
         }
 )
 
+setAs("GAlignmentsList", "GAlignmentPairs", function(from) {
+          ga <- unlist(from[mcols(from)$mate_status != "unmated"])
+          first <- c(TRUE, FALSE)
+          last <- c(FALSE, TRUE)
+          isProperPair <- if (!is.null(mcols(ga)$flag)) {
+              bamFlagTest(mcols(ga)$flag[first], "isProperPair")
+          } else {
+              TRUE
+          }
+          GAlignmentPairs(ga[first], ga[last],
+                          isProperPair=isProperPair,
+                          names=names(ga)[first])
+      })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting.
