@@ -90,10 +90,9 @@
 ### Puts NAs last.
 .getCharacterOrderAndGroupSizes <- function(x)
 {
-    x2 <- match(x, x,
-                nomatch=.Machine$integer.max,
-                incomparables=NA_character_)
-    xo <- S4Vectors:::orderInteger(x2)
+    x2 <- match(x, x, nomatch=.Machine$integer.max,
+                      incomparables=NA_character_)
+    xo <- base::order(x2)
     ox2 <- Rle(x2[xo])
     group.sizes <- runLength(ox2)
     ngroup <- length(group.sizes)
@@ -270,7 +269,7 @@ findMateAlignment <- function(x)
         subject_low2high <- S4Vectors:::reverseSelfmatchMapping(
                                 high2low(subject))
         extra_hits <- subject_low2high[subject_hits0]
-        query_nhits <- 1L + elementLengths(extra_hits)
+        query_nhits <- 1L + elementNROWS(extra_hits)
         query_hits <- rep.int(query_hits0, query_nhits)
         subject_hits <- integer(length(query_hits))
         idx0 <- cumsum(c(1L, query_nhits[-length(query_nhits)]))
@@ -278,7 +277,8 @@ findMateAlignment <- function(x)
         subject_hits[-idx0] <- unlist(extra_hits,
                                       recursive=FALSE, use.names=FALSE)
     }
-    Hits(query_hits, subject_hits, length(query), length(subject))
+    Hits(query_hits, subject_hits, length(query), length(subject),
+         sort.by.query=TRUE)
 }
 
 ### Use to find self matches in 'x'. Twice faster than
@@ -290,9 +290,9 @@ findMateAlignment <- function(x)
     xo <- xo_and_GS$xo
     GS <- xo_and_GS$group.sizes
     ans <- S4Vectors:::makeAllGroupInnerHits(GS, hit.type=1L)
-    ans@queryHits <- xo[ans@queryHits]
-    ans@subjectHits <- xo[ans@subjectHits]
-    ans@queryLength <- ans@subjectLength <- length(x)
+    ans@from <- xo[ans@from]
+    ans@to <- xo[ans@to]
+    ans@nLnode <- ans@nRnode <- length(x)
     ans
 }
 
