@@ -209,8 +209,7 @@ setGeneric("readGAlignmentPairs", signature="file",
 
     ## Split and order the pairs by ascending start position of the first mate.
     idx1 <- which(as.logical(is_first_mate))
-    oo1 <- S4Vectors:::orderIntegerPairs(as.integer(gal@seqnames)[idx1],
-                                         gal@start[idx1])
+    oo1 <- orderIntegerPairs(as.integer(gal@seqnames)[idx1], gal@start[idx1])
     idx1 <- idx1[oo1]
     idx2 <- which(as.logical(is_last_mate))[oo1]
     ans_first <- gal[idx1]
@@ -421,32 +420,3 @@ setMethod("readGappedReads", "character",
     }
 )
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Old stuff (deprecated & defunct)
-###
-
-readGAlignmentsFromBam <- function(...)
-{
-    .Defunct("readGAlignments")
-    readGAlignments(...)
-}
-
-readGAlignmentPairsFromBam <- function(...)
-{
-    .Defunct("readGAlignmentPairs")
-    readGAlignmentPairs(...)
-}
-
-readGAlignmentsListFromBam <- function(...)
-{
-    .Defunct("readGAlignmentsList")
-    readGAlignmentsList(...)
-}
-
-readGappedReadsFromBam <- function(...)
-{
-    .Defunct("readGappedReads")
-    readGappedReads(...)
-}
-
